require "ljsc/init"

local sc = require "screen"
local png = require "png"
local util = require "util"

if not sc.init(1, 320, 240) then sc.init(0, 320, 240) end

bcol = sc.rgb(0.5, 0.5, 1)
sc.cls(bcol)

dir_x = {0.924, 0.383, -0.383, -0.924, -0.924, -0.383, 0.383, 0.924 }
dir_y = {0.383, 0.924, 0.924, 0.383, -0.383, -0.924, -0.924, -0.383 }

ofs_x = math.floor(sc.width / 4)
ofs_y = math.floor(sc.height / 4)
range_x = math.floor(sc.width * 1.5)
range_y = math.floor(sc.height * 1.5)

--  0: right, 1: up-right, 2: up, 3: up-left,
--  4: left, 5: down-left, 6: down, 7: down-right
sc.patpng(4, "fish.png")
sc.patpng(3, "fish2.png")
w, h, t4 = sc.patget(4)
w, h, t3 = sc.patget(3)
function rotate_left(t, w, h)
  local t2 = {}
  for y = 0, h - 1 do
    for x = 0, w - 1 do
      t2[x + y * h + 1] = t[(h - 1 - y) + x * h + 1]
    end
  end
  return t2
end
function flip_horizontal(t, w, h)
  local t2 = {}
  for y = 0, h - 1 do
    for x = 0, w - 1 do
      t2[x + y * h + 1] = t[w - 1 - x + y * h + 1]
    end
  end
  return t2
end
t0 = flip_horizontal(t4, w, h)
t1 = flip_horizontal(t3, w, h)
t2 = rotate_left(t0, w, h)
t5 = rotate_left(t3, w, h)
t6 = rotate_left(t4, w, h)
t7 = flip_horizontal(t5, w, h)
sc.patdef(0, w, h, t0)
sc.patdef(1, w, h, t1)
sc.patdef(2, w, h, t2)
sc.patdef(5, w, h, t5)
sc.patdef(6, w, h, t6)
sc.patdef(7, w, h, t7)

--[[
sc.patdraw(0, 300, 220)
sc.flush()
util.sleep(10)
os.exit(0)
--]]

function collide(x1, y1, x2, y2, r)
  local x = x2 - x1
  local y = y2 - y1
  if x * x + y * y < r * r then
    local rad = math.floor((math.atan2(y, x) + 3.1416 * (0.75 + (math.random() * 0.5))) * 4 / 3.1416)
    return (rad + 8) % 8
  else
    return -1
  end
end

nfish = 8
fish = {}
for i = 1, nfish do
  fish[i] = {}
  flag = true
  repeat
    x = math.floor(math.random() * range_x) - ofs_x
    y = math.floor(math.random() * range_y) - ofs_y
    flag = true
    for j = 1, i - 1 do
      x1 = fish[j].x
      y1 = fish[j].y
      if collide(x, y, x1, y1, 32) >= 0 then
        flag = false
        break
      end
    end
  until flag
  fish[i].x = x
  fish[i].y = y
  fish[i].speed = 8
  fish[i].direction = math.floor(math.random() * 8)
end

function proceed(n)
  local x, y, x1, y1, newdir
  x = fish[n].x
  y = fish[n].y
  for j = 1, nfish do
    if n ~= j then
      x1 = fish[j].x
      y1 = fish[j].y
      local r = collide(x, y, x1, y1, 40)
      if r >= 0 then
        newdir = r
        break
      end
    end
  end
  sc.paterase(fish[n].direction, x, y, bcol)
  if newdir == nil then newdir = fish[n].direction end
  local speed = fish[n].speed
  local dx = dir_x[newdir + 1]
  local dy = dir_y[newdir + 1]
  x = math.floor(x + dx * speed + ofs_x)
  y = math.floor(y + dy * speed + ofs_y)
  if x < 0 or x >= range_x then
    newdir = (12 - newdir) % 8
    if x < 0 then x = 0 else x = range_x end
  end
  if y < 0 or y >= range_y then
    newdir = (8 - newdir) % 8
    if y < 0 then y = 0 else y = range_y end
  end
  x = x - ofs_x
  y = y - ofs_y
  fish[n].direction = newdir
  sc.patdraw(newdir, x, y)
  fish[n].x = x
  fish[n].y = y
end

while true do
  local tbase = util.now()
  for n = 1, nfish do
    proceed(n)
  end
  sc.flush()
  local dt = tbase + 0.05 - util.now()
  if dt > 0 then
    util.sleep(dt)
  end
end
